@@ -13,7 +13,7 @@
 ##############################################################################
 """HTTP Publication
 
-$Id: http.py 100294 2009-05-23 21:06:18Z shane $
+$Id: http.py 100362 2009-05-25 16:40:17Z shane $
 """
 
 __docformat__ = 'restructuredtext'
@@ -27,10 +27,7 @@ import zope.component
 
 from zope.app.publication.zopepublication import ZopePublication
 
-from zope.publisher.interfaces.http import IMethodNotAllowed
-from zope.deprecation import deprecated
-deprecated(
-    "IMethodNotAllowed", "moved to zope.publisher.interfaces")
+from zope.publisher.interfaces.http import IMethodNotAllowed #BBB import
 
 
 class BaseHTTPPublication(ZopePublication):
