@@ -13,7 +13,7 @@
 ##############################################################################
 """Zope Publication Tests
 
-$Id: test_zopepublication.py 91191 2008-09-16 15:06:24Z asmith $
+$Id: test_zopepublication.py 102692 2009-08-11 22:34:23Z gary $
 """
 import unittest
 import sys
@@ -43,14 +43,13 @@ from zope.location.interfaces import ILocation
 from zope.app.testing.placelesssetup import PlacelessSetup
 from zope.app.testing import setup, ztapi
 
-from zope.app.security.principalregistry import principalRegistry
-from zope.app.security.interfaces import IUnauthenticatedPrincipal, IPrincipal
+from zope.authentication.interfaces import IAuthentication
+from zope.authentication.interfaces import IFallbackUnauthenticatedPrincipal
+from zope.authentication.interfaces import IUnauthenticatedPrincipal
+from zope.security.interfaces import IPrincipal
+from zope.principalregistry.principalregistry import principalRegistry
 from zope.app.publication.zopepublication import ZopePublication
 from zope.app.folder import Folder, rootFolder
-from zope.app.security.interfaces import IAuthenticationUtility
-from zope.app.security.interfaces import IAuthentication
-from zope.app.security.interfaces import IFallbackUnauthenticatedPrincipal
-from zope.app.security.principalregistry import principalRegistry
 
 
 class Principal(object):
@@ -269,7 +268,7 @@ class ZopePublicationErrorHandling(BasePublicationTests):
                                  type=self.presentation_type)
         view_text = 'You had a conflict error'
 
-        from zope.app.exception.interfaces import ISystemErrorView
+        from zope.browser.interfaces import ISystemErrorView
         class MyView:
             implements(ISystemErrorView)
             def __init__(self, context, request):
@@ -395,7 +394,7 @@ class ZopePublicationErrorHandling(BasePublicationTests):
         class FooError(Exception):
             pass
 
-        last_txn_info = self.storage.undoInfo()[0]
+        last_txn_info = self.storage.lastTransaction()
         try:
             raise FooError
         except FooError:
@@ -404,7 +403,7 @@ class ZopePublicationErrorHandling(BasePublicationTests):
             self.object, self.request, sys.exc_info(), retry_allowed=False)
 
         # assert that the last transaction is NOT our transaction
-        new_txn_info = self.storage.undoInfo()[0]
+        new_txn_info = self.storage.lastTransaction()
         self.assertEqual(last_txn_info, new_txn_info)
 
         # instead, we expect a message in our logging utility
@@ -415,6 +414,45 @@ class ZopePublicationErrorHandling(BasePublicationTests):
         self.assert_(isinstance(error_info[1], FooError))
         self.assert_(request is self.request)
 
+    def testLogBeforeAbort(self):
+        # If we get an exception, and then (a catastrophe, but one that has
+        # been experienced) transaction.abort fails, we really want to know
+        # what happened before that abort.
+        # (Set up:)
+        zope.component.provideUtility(ErrorReportingUtility())
+        abort = transaction.abort
+        class AbortError(Exception):
+            pass
+        class AnEarlierError(Exception):
+            pass
+        def faux_abort():
+            raise AbortError
+        try:
+            raise AnEarlierError()
+        except AnEarlierError:
+            pass
+        transaction.abort = faux_abort
+        try:
+            # (Test:)
+            try:
+                self.publication.handleException(
+                    self.object, self.request, sys.exc_info(),
+                    retry_allowed=False)
+            except AbortError:
+                pass
+            else:
+                self.fail('Aborting should have failed')
+            # we expect a message in our logging utility
+            error_log = zope.component.getUtility(IErrorReportingUtility)
+            self.assertEqual(len(error_log.exceptions), 1)
+            error_info, request = error_log.exceptions[0]
+            self.assertEqual(error_info[0], AnEarlierError)
+            self.failUnless(isinstance(error_info[1], AnEarlierError))
+            self.failUnless(request is self.request)
+        finally:
+            # (Tear down:)
+            transaction.abort = abort
+
 
 class ZopePublicationTests(BasePublicationTests):
 
@@ -446,11 +484,11 @@ class ZopePublicationTests(BasePublicationTests):
         f1 = app['f1']
         f1['f2'] = Folder()
         sm1 = setup.createSiteManager(f1)
-        setup.addUtility(sm1, '', IAuthenticationUtility, AuthUtility1())
+        setup.addUtility(sm1, '', IAuthentication, AuthUtility1())
 
         f2 = f1['f2']
         sm2 = setup.createSiteManager(f2)
-        setup.addUtility(sm2, '', IAuthenticationUtility, AuthUtility2())
+        setup.addUtility(sm2, '', IAuthentication, AuthUtility2())
         transaction.commit()
 
         from zope.app.container.interfaces import ISimpleReadContainer
@@ -487,10 +525,10 @@ class ZopePublicationTests(BasePublicationTests):
         # we just need a change in the database to make the
         # transaction notable in the undo log
         root['foo'] = object()
-        last_txn_info = self.storage.undoInfo()[0]
+        last_txn_info = self.storage.lastTransaction()
         self.publication.afterCall(self.request, self.object)
         self.assert_(txn is not transaction.get())
-        new_txn_info = self.storage.undoInfo()[0]
+        new_txn_info = self.storage.lastTransaction()
         self.failIfEqual(last_txn_info, new_txn_info)
 
     def testDoomedTransaction(self):
@@ -500,13 +538,13 @@ class ZopePublicationTests(BasePublicationTests):
         # we just need a change in the database to make the
         # transaction notable in the undo log
         root['foo'] = object()
-        last_txn_info = self.storage.undoInfo()[0]
+        last_txn_info = self.storage.lastTransaction()
         # doom the transaction
         txn.doom()
         self.publication.afterCall(self.request, self.object)
         # assert that we get a new transaction
         self.assert_(txn is not transaction.get())
-        new_txn_info = self.storage.undoInfo()[0]
+        new_txn_info = self.storage.lastTransaction()
         # No transaction should be committed
         self.assertEqual(last_txn_info, new_txn_info)
 
@@ -519,6 +557,19 @@ class ZopePublicationTests(BasePublicationTests):
         ztapi.provideAdapter(ILocation, IPhysicallyLocatable,
                              LocationPhysicallyLocatable)
 
+        def get_txn_info():
+            if hasattr(self.storage, 'iterator'):
+                # ZODB 3.9
+                txn_id = self.storage.lastTransaction()
+                txn = list(self.storage.iterator(txn_id, txn_id))[0]
+                txn_info = dict(location=txn.extension['location'],
+                                user_name=txn.user,
+                                request_type=txn.extension['request_type'])
+            else:
+                # ZODB 3.8
+                txn_info = self.storage.undoInfo()[0]
+            return txn_info
+
         root = self.db.open().root()
         root['foo'] = foo = LocatableObject()
         root['bar'] = bar = LocatableObject()
@@ -534,7 +585,7 @@ class ZopePublicationTests(BasePublicationTests):
         expected_request = IRequest.__module__ + '.' + IRequest.getName()
 
         self.publication.afterCall(self.request, bar)
-        txn_info = self.storage.undoInfo()[0]
+        txn_info = get_txn_info()
         self.assertEqual(txn_info['location'], expected_path)
         self.assertEqual(txn_info['user_name'], expected_user)
         self.assertEqual(txn_info['request_type'], expected_request)
@@ -542,6 +593,7 @@ class ZopePublicationTests(BasePublicationTests):
         # also, assert that we still get the right location when
         # passing an instance method as object.
         self.publication.afterCall(self.request, bar.foo)
+        txn_info = get_txn_info()
         self.assertEqual(txn_info['location'], expected_path)
 
     def testSiteEvents(self):
