@@ -11,10 +11,6 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-"""Zope publication
-
-$Id: zopepublication.py 101160 2009-06-20 10:59:31Z jim $
-"""
 __docformat__ = 'restructuredtext'
 
 import sys
@@ -25,17 +21,21 @@ from ZODB.POSException import ConflictError
 import transaction
 
 import zope.component
+from zope.component import queryMultiAdapter
 from zope.event import notify
 from zope.security.interfaces import Unauthorized
 from zope.interface import implements, providedBy
 from zope.publisher.publish import mapply
-from zope.publisher.interfaces import Retry, IExceptionSideEffects, IHeld
-from zope.publisher.interfaces import IRequest, IPublication
+from zope.publisher.interfaces import IExceptionSideEffects, IHeld
+from zope.publisher.interfaces import IPublication, IPublishTraverse, IRequest
+from zope.publisher.interfaces import NotFound, Retry
 from zope.security.management import newInteraction, endInteraction
 from zope.security.checker import ProxyFactory
 from zope.security.proxy import removeSecurityProxy
 from zope.traversing.interfaces import IPhysicallyLocatable
 from zope.traversing.interfaces import IEtcNamespace
+from zope.traversing.interfaces import TraversalError
+from zope.traversing.namespace import namespaceLookup, nsParse
 from zope.location import LocationProxy
 from zope.error.interfaces import IErrorReportingUtility
 
@@ -43,7 +43,6 @@ import zope.authentication.interfaces
 from zope.browser.interfaces import ISystemErrorView
 from zope.app.publication.interfaces import BeforeTraverseEvent
 from zope.app.publication.interfaces import EndRequestEvent
-from zope.traversing.publicationtraverse import PublicationTraverse
 from zope.publisher.defaultview import queryDefaultViewName
 from zope.authentication.interfaces import IUnauthenticatedPrincipal
 from zope.authentication.interfaces import IFallbackUnauthenticatedPrincipal
@@ -67,7 +66,7 @@ class Cleanup(object):
                 "Cleanup without request close")
             self._f()
 
-class ZopePublication(PublicationTraverse):
+class ZopePublication(object):
     """Base Zope publication specification."""
     implements(IPublication)
 
@@ -171,6 +170,36 @@ class ZopePublication(PublicationTraverse):
 
         return self.proxy(app)
 
+    def traverseName(self, request, ob, name):
+        nm = name # the name to look up the object with
+
+        if name and name[:1] in '@+':
+            # Process URI segment parameters.
+            ns, nm = nsParse(name)
+            if ns:
+                try:
+                    ob2 = namespaceLookup(ns, nm, ob, request)
+                except TraversalError:
+                    raise NotFound(ob, name)
+
+                return self.proxy(ob2)
+
+        if nm == '.':
+            return ob
+
+        if IPublishTraverse.providedBy(ob):
+            ob2 = ob.publishTraverse(request, nm)
+        else:
+            # self is marker
+            adapter = queryMultiAdapter((ob, request), IPublishTraverse,
+                                        default=self)
+            if adapter is not self:
+                ob2 = adapter.publishTraverse(request, nm)
+            else:
+                raise NotFound(ob, name, request)
+
+        return self.proxy(ob2)
+
     def callObject(self, request, ob):
         return mapply(ob, request.getPositionalArguments(), request)
 
@@ -256,7 +285,12 @@ class ZopePublication(PublicationTraverse):
     def handleException(self, object, request, exc_info, retry_allowed=True):
         # This transaction had an exception that reached the publisher.
         # It must definitely be aborted.
-        transaction.abort()
+        try:
+            transaction.abort()
+        except:
+            # Hm, a catastrophe.  We might want to know what preceded it.
+            self._logErrorWithErrorReportingUtility(object, request, exc_info)
+            raise
 
         # Reraise Retry exceptions for the publisher to deal with.
         if retry_allowed and isinstance(exc_info[1], Retry):
@@ -276,7 +310,7 @@ class ZopePublication(PublicationTraverse):
         # handling determine whether a retry is allowed or not?
         # Assume not for now.
 
-        # Record the error with the ErrorReportingUtility
+        # Record the error with the ErrorReportingUtility.
         self._logErrorWithErrorReportingUtility(object, request, exc_info)
 
         response = request.response
